@@ -2,7 +2,7 @@
 
 void hier()
 {
-  c("/","etc","dnsroots.global",-1,-1,0644);
+//  c("/","etc","dnsroots.global",-1,-1,0644);
 
   h(auto_home,-1,-1,02755);
   d(auto_home,"bin",-1,-1,02755);
@@ -38,4 +38,5 @@ void hier()
   c(auto_home,"bin","dnsqr",-1,-1,0755);
   c(auto_home,"bin","dnsq",-1,-1,0755);
   c(auto_home,"bin","dnstrace",-1,-1,0755);
+  c(auto_home,"bin","dnstracesort",-1,-1,0755);
 }
